@@ -22,7 +22,6 @@
 
 #include "postgres.h"
 
-#include "miscadmin.h"
 #include "nodes/plannodes.h"
 #include "nodes/relation.h"
 #include "utils/datum.h"
@@ -2895,7 +2894,7 @@ _copyReassignOwnedStmt(ReassignOwnedStmt *from)
 	ReassignOwnedStmt *newnode = makeNode(ReassignOwnedStmt);
 
 	COPY_NODE_FIELD(roles);
-	COPY_STRING_FIELD(newrole);
+	COPY_SCALAR_FIELD(newrole);
 
 	return newnode;
 }
@@ -3015,9 +3014,6 @@ copyObject(void *from)
 	if (from == NULL)
 		return NULL;
 
-	/* Guard against stack overflow due to overly complex expressions */
-	check_stack_depth();
-
 	switch (nodeTag(from))
 	{
 			/*
