@@ -1,19 +1,49 @@
 /*
  * psql - the PostgreSQL interactive terminal
  *
- * Copyright (c) 2000-2005, PostgreSQL Global Development Group
+ * Copyright (c) 2000-2008, PostgreSQL Global Development Group
  *
- * $PostgreSQL: pgsql/src/bin/psql/large_obj.c,v 1.35.4.1 2006/05/21 19:57:39 momjian Exp $
+ * $PostgreSQL: pgsql/src/bin/psql/large_obj.c,v 1.50 2008/01/01 19:45:56 momjian Exp $
  */
 #include "postgres_fe.h"
 #include "large_obj.h"
 
-#include "libpq-fe.h"
 
 #include "settings.h"
-#include "variables.h"
 #include "common.h"
-#include "print.h"
+
+static void
+print_lo_result(const char *fmt,...)
+__attribute__((format(printf, 1, 2)));
+
+static void
+print_lo_result(const char *fmt,...)
+{
+	va_list		ap;
+
+	if (!pset.quiet)
+	{
+		if (pset.popt.topt.format == PRINT_HTML)
+			fputs("<p>", pset.queryFout);
+
+		va_start(ap, fmt);
+		vfprintf(pset.queryFout, fmt, ap);
+		va_end(ap);
+
+		if (pset.popt.topt.format == PRINT_HTML)
+			fputs("</p>\n", pset.queryFout);
+		else
+			fputs("\n", pset.queryFout);
+	}
+
+	if (pset.logfile)
+	{
+		va_start(ap, fmt);
+		vfprintf(pset.logfile, fmt, ap);
+		va_end(ap);
+		fputs("\n", pset.logfile);
+	}
+}
 
 
 /*
@@ -70,8 +100,7 @@ finish_lo_xact(const char *operation, bool own_transaction)
 {
 	PGresult   *res;
 
-	if (own_transaction &&
-		GetVariableBool(pset.vars, "AUTOCOMMIT"))
+	if (own_transaction && pset.autocommit)
 	{
 		/* close out our own xact */
 		if (!(res = PSQLexec("COMMIT", false)))
@@ -94,8 +123,7 @@ fail_lo_xact(const char *operation, bool own_transaction)
 {
 	PGresult   *res;
 
-	if (own_transaction &&
-		GetVariableBool(pset.vars, "AUTOCOMMIT"))
+	if (own_transaction && pset.autocommit)
 	{
 		/* close out our own xact */
 		res = PSQLexec("ROLLBACK", false);
@@ -120,10 +148,13 @@ do_lo_export(const char *loid_arg, const char *filename_arg)
 	if (!start_lo_xact("\\lo_export", &own_transaction))
 		return false;
 
+	SetCancelConn();
 	status = lo_export(pset.db, atooid(loid_arg), filename_arg);
+	ResetCancelConn();
+
+	/* of course this status is documented nowhere :( */
 	if (status != 1)
-	{							/* of course this status is documented
-								 * nowhere :( */
+	{
 		fputs(PQerrorMessage(pset.db), stderr);
 		return fail_lo_xact("\\lo_export", own_transaction);
 	}
@@ -131,7 +162,7 @@ do_lo_export(const char *loid_arg, const char *filename_arg)
 	if (!finish_lo_xact("\\lo_export", own_transaction))
 		return false;
 
-	fprintf(pset.queryFout, "lo_export\n");
+	print_lo_result("lo_export");
 
 	return true;
 }
@@ -148,13 +179,15 @@ do_lo_import(const char *filename_arg, const char *comment_arg)
 	PGresult   *res;
 	Oid			loid;
 	char		oidbuf[32];
-	unsigned int i;
 	bool		own_transaction;
 
 	if (!start_lo_xact("\\lo_import", &own_transaction))
 		return false;
 
+	SetCancelConn();
 	loid = lo_import(pset.db, filename_arg);
+	ResetCancelConn();
+
 	if (loid == InvalidOid)
 	{
 		fputs(PQerrorMessage(pset.db), stderr);
@@ -171,16 +204,9 @@ do_lo_import(const char *filename_arg, const char *comment_arg)
 		cmdbuf = malloc(slen * 2 + 256);
 		if (!cmdbuf)
 			return fail_lo_xact("\\lo_import", own_transaction);
-		sprintf(cmdbuf,
-				"COMMENT ON LARGE OBJECT %u IS '",
-				loid);
+		sprintf(cmdbuf, "COMMENT ON LARGE OBJECT %u IS '", loid);
 		bufptr = cmdbuf + strlen(cmdbuf);
-		for (i = 0; i < slen; i++)
-		{
-			if (comment_arg[i] == '\'' || comment_arg[i] == '\\')
-				*bufptr++ = comment_arg[i];
-			*bufptr++ = comment_arg[i];
-		}
+		bufptr += PQescapeStringConn(pset.db, bufptr, comment_arg, slen, NULL);
 		strcpy(bufptr, "'");
 
 		if (!(res = PSQLexec(cmdbuf, false)))
@@ -196,7 +222,8 @@ do_lo_import(const char *filename_arg, const char *comment_arg)
 	if (!finish_lo_xact("\\lo_import", own_transaction))
 		return false;
 
-	fprintf(pset.queryFout, "lo_import %u\n", loid);
+	print_lo_result("lo_import %u", loid);
+
 	sprintf(oidbuf, "%u", loid);
 	SetVariable(pset.vars, "LASTOID", oidbuf);
 
@@ -219,7 +246,10 @@ do_lo_unlink(const char *loid_arg)
 	if (!start_lo_xact("\\lo_unlink", &own_transaction))
 		return false;
 
+	SetCancelConn();
 	status = lo_unlink(pset.db, loid);
+	ResetCancelConn();
+
 	if (status == -1)
 	{
 		fputs(PQerrorMessage(pset.db), stderr);
@@ -229,7 +259,7 @@ do_lo_unlink(const char *loid_arg)
 	if (!finish_lo_xact("\\lo_unlink", own_transaction))
 		return false;
 
-	fprintf(pset.queryFout, "lo_unlink %u\n", loid);
+	print_lo_result("lo_unlink %u", loid);
 
 	return true;
 }
@@ -249,11 +279,12 @@ do_lo_list(void)
 	printQueryOpt myopt = pset.popt;
 
 	snprintf(buf, sizeof(buf),
-			 "SELECT loid as \"ID\",\n"
-			 "  pg_catalog.obj_description(loid, 'pg_largeobject') as \"%s\"\n"
+			 "SELECT loid as \"%s\",\n"
+		   "  pg_catalog.obj_description(loid, 'pg_largeobject') as \"%s\"\n"
 			 "FROM (SELECT DISTINCT loid FROM pg_catalog.pg_largeobject) x\n"
 			 "ORDER BY 1",
-			 gettext("Description"));
+			 gettext_noop("ID"),
+			 gettext_noop("Description"));
 
 	res = PSQLexec(buf, false);
 	if (!res)
@@ -261,9 +292,10 @@ do_lo_list(void)
 
 	myopt.topt.tuples_only = false;
 	myopt.nullPrint = NULL;
-	myopt.title = gettext("Large objects");
+	myopt.title = _("Large objects");
+	myopt.trans_headers = true;
 
-	printQuery(res, &myopt, pset.queryFout);
+	printQuery(res, &myopt, pset.queryFout, pset.logfile);
 
 	PQclear(res);
 	return true;
